<?php
/**
 * Dklab_Cache_Backend_TagEmuWrapper: tag wrapper for any Zend_Cache backend.
 * 
 * Implements tags. Tags are emulated via keys: unfortunately this 
 * increases the data read cost (the more tags are assigned to a key,
 * the more read cost becomes).
 *
 * $Id$
 */
require_once "Zend/Cache/Backend/Interface.php";
 
class Dklab_Cache_Backend_TagEmuWrapper implements Zend_Cache_Backend_Interface 
{
    private $_backend = null;
    
    
    public function __construct(Zend_Cache_Backend_Interface $backend)
    {
        $this->_backend = $backend;
    }
    
    
    public function setDirectives($directives)
    {
        return $this->_backend->setDirectives($directives);
    }
    
    
    public function load($id, $doNotTestCacheValidity = false)
    {
        // Data is saved in form of: array(tagsArray, anyData).
        $serialized = $this->_backend->load($id, $doNotTestCacheValidity);
        if ($serialized === false) {
            return false;
        }
        $combined = unserialize($serialized);
        if (!is_array($combined)) {
            return false;
        } 
        // Test if all tags are still not removed/expired.
        if (is_array($combined[0])) {
            foreach ($combined[0] as $tag) {
                if (!$this->_backend->test($this->_mangleTag($tag))) {
                    return false;
                }
            }
        }
        return $combined[1];
    }
    
    
    public function save($data, $id, $tags = array(), $specificLifetime = false)
    {
        // Data is saved in form of: array(tagsArray, anyData).
        $combined = array($tags, $data);
        // Save/update tags as usual infinite keys with value "".
        if (is_array($combined[0])) {
            foreach ($combined[0] as $tag) {
                $this->_backend->save(1, $this->_mangleTag($tag), array(), null);
            }
        }
        $serialized = serialize($combined);
        return $this->_backend->save($serialized, $id, array(), $specificLifetime);
    }
    

    public function clean($mode = Zend_Cache::CLEANING_MODE_ALL, $tags = array())
    {
        if ($mode == Zend_Cache::CLEANING_MODE_MATCHING_TAG) {
            if (is_array($tags)) {
                foreach ($tags as $tag) {
                    $this->_backend->remove($this->_mangleTag($tag));
                }
            }
        } else {
            return $this->_backend->clean($mode, $tags);
        }
    }

    
    public function test($id)
    {
        // Data is saved in form of: array(tagsArray, anyData).
        $serialized = $this->_backend->load($id);
        if ($serialized === false) {
            return false;
        }
        $combined = unserialize($serialized);
        if (!is_array($combined)) {
            return false;
        } 
        // Test if all tags are still not removed/expired.
        if (is_array($combined[0])) {
            foreach ($combined[0] as $tag) {
                if (!$this->_backend->test($this->_mangleTag($tag))) {
                    return false;
                }
            }
        }
        return true;
    }
    
    
    public function remove($id)
    {
        return $this->_backend->remove($id);
    }
    
    
    
    /**
     * Mangles the name to deny intersection of tag keys & data keys.
     * Mangled tag names are NOT saved in memcache $combined[0] value,
     * mangling is always performed on-demand (to same some space).
     * 
     * @param string $tag    Tag name to mangle.
     * @return string        Mangled tag name.
     */
    private function _mangleTag($tag)
    {
        return __CLASS__ . "_" . $tag;
    }
}
